@@ -8,7 +8,7 @@ from datachain.error import (
     ProjectNotFoundError,
 )
 from datachain.lib.namespaces import create as create_namespace
-from datachain.lib.namespaces import delete as delete_namespace
+from datachain.lib.namespaces import delete_namespace
 from datachain.lib.namespaces import get as get_namespace
 from datachain.lib.projects import get as get_project
 from datachain.lib.projects import ls as ls_projects
